module Messages where

import Terms
import Rules
import Trie
import TPTP
import Horn



unknownMessage str = "moca: unknown option '" ++ str ++ "'.\n"

helpMessage = "Moca v1.0\n"++ 
            "Usage: ./moca.sh <options> <file>\n\n" ++ 
              "\t-help:\t\tDisplay this list of options\n" ++
              "\t--help:\t\tDisplay this list of options\n" 

errorMessage_SpecifyFilename = "Please specify filename. For Instance \n  $ ./moca.sh problem.trs"

errorMessage_NoZ3 = "Z3 cannot be found."

resultForINF_Infeasible :: [HornClause Atom] -> ES -> ES -> OTRS -> String -> String
resultForINF_Infeasible hc es gs otrs remark = "YES\n\n" ++ 
  "proof:\n" ++ 
  "The input problem is infeasible because\n\n" ++
  "[1] the following set of Horn clauses is satisfiable:\n\n"++
  showHornClause hc ++ "\n\n" ++
  "This holds because\n\n" ++
  "[2] the following E does not entail the following G (Claessen-Smallbone's transformation (2018)):\n\n" ++
  "E:\n" ++ showES es ++ "\nG:\n" ++ showES gs ++ "\n\n" ++
  "This holds because\n\n" ++
  "[3] the following ground-complete ordered TRS entails E but does not entail G:\n" ++
  showOTRS otrs ++ "\n" ++
  remark





resultForINF_Maybe :: OTRS -> String -> String
resultForINF_Maybe otrs remark = "MAYBE\n" ++ remark

resultForTPTP_SAT :: OTRS -> String -> String
resultForTPTP_SAT otrs remark = "# SZS status Satisfiable\n"   ++ showOTRS otrs ++ "\n" ++ remark

resultForTPTP_UNSAT :: OTRS -> String -> String
resultForTPTP_UNSAT otrs remark = "# SZS status Unsatisfiable\n" ++ showOTRS otrs ++ "\n" ++ remark


resultForTPTP_EmptyInput :: String 
resultForTPTP_EmptyInput = "# SZS status Satisfiable\n"
resultForINF_EmptyInput :: String 
resultForINF_EmptyInput = "ERROR\nNo condition."