module ParserForZ3 where

import Terms
import Substitutions
import Rules
import Text.ParserCombinators.Parsec

-- [DATA] Term
data IntOrBool = I Int | B Bool deriving Eq
instance Show IntOrBool where
  show (I n) = show n
  show (B b) = show b

-- [LITERALS]
lSat   = "sat"

number :: Parser Int
number = do ds <- many1 digit
            return (read ds)

keyword :: String -> Parser ()
keyword s = do { string s; spaces; return () }

symbol :: Parser String
symbol = do { x <- many1 (alphaNum); spaces; return x }

positive :: Parser Int
positive = do { x <- number;
                return x }
negative :: Parser Int
negative = do { keyword "(- ";  x <- number; keyword ")";
                return (-x) }
integer :: Parser Int
integer = try positive <|> negative

true :: Parser Bool
true = do { keyword "true"; 
            return True }
false :: Parser Bool
false = do { keyword "false"; 
             return False }
bool :: Parser Bool
bool = try true <|> false

intResult :: Parser (String, IntOrBool)
intResult = do { keyword "("; f <- symbol; spaces; n <- integer; keyword ")";
                 return (f, I n) }

boolResult :: Parser (String, IntOrBool)
boolResult = do { keyword "("; f <- symbol; spaces; b <- bool; keyword ")";
                  return (f, B b) }

result :: Parser (String, IntOrBool)
result = try intResult <|> boolResult

results :: Parser [(String, IntOrBool)]
results = do { keyword lSat; spaces;  keyword "("; ibs <- many result;  keyword ")"; 
               return ibs }

getResults :: String -> Maybe [(String, IntOrBool)]
getResults s | Right rs <- parse results "" s  = Just rs
             | otherwise                       = Nothing

getIntResults :: String -> Maybe [(String, Int)]
getIntResults s | Just rs <- getResults s  = Just [(s, n) | (s, I n) <- rs]
                | otherwise                = Nothing

getBoolResults :: String -> Maybe [(String, Bool)]
getBoolResults s | Just rs <- getResults s  = Just [(s, b) | (s, B b) <- rs]
                 | otherwise                = Nothing



